from unittest.mock import ANY, Mock, call, patch

from allianceauth.utils.testing import NoSocketsTestCase

from drifters.tasks import garbage_collection


class TestTasks(NoSocketsTestCase):
    @staticmethod
    def _make_wormhole(**overrides) -> Mock:
        wormhole = Mock()
        wormhole.id = overrides.pop("id", 1)
        wormhole.lifetime = overrides.pop("lifetime", "Fresh")
        wormhole.set_archived = overrides.pop("set_archived", Mock(return_value=True))
        wormhole.set_eol = overrides.pop("set_eol", Mock(return_value=True))

        for key, value in overrides.items():
            setattr(wormhole, key, value)

        return wormhole

    @staticmethod
    def _make_queryset(items: list[Mock]) -> Mock:
        queryset = Mock()
        queryset.__iter__ = Mock(return_value=iter(items))
        queryset.count.return_value = len(items)
        queryset.exists.return_value = bool(items)
        queryset.all.return_value = items
        return queryset

    @patch("drifters.tasks.Wormhole")
    def test_garbage_collection_archives_and_sets_eol(self, mock_wormhole) -> None:
        old_hole = self._make_wormhole(id=1, lifetime="Fresh")
        old_eol_hole = self._make_wormhole(id=2, lifetime="EOL")
        decaying_hole = self._make_wormhole(id=3, lifetime="Decaying")

        mock_wormhole.objects.filter.side_effect = [
            self._make_queryset([old_hole]),
            self._make_queryset([old_eol_hole]),
            self._make_queryset([decaying_hole]),
        ]

        garbage_collection()

        old_hole.set_archived.assert_called_once_with()
        old_eol_hole.set_archived.assert_called_once_with()
        decaying_hole.set_eol.assert_called_once_with()
        self.assertEqual(mock_wormhole.objects.filter.call_count, 3)
        self.assertEqual(
            mock_wormhole.objects.filter.call_args_list,
            [
                call(created_at__lte=ANY),
                call(lifetime=mock_wormhole.Lifetime.EOL, eol_changed_at__lte=ANY),
                call(
                    lifetime__in=[mock_wormhole.Lifetime.DECAY, mock_wormhole.Lifetime.FRESH],
                    created_at__lte=ANY,
                ),
            ],
        )

    @patch("drifters.tasks.Wormhole")
    def test_garbage_collection_with_no_wormholes(self, mock_wormhole) -> None:
        mock_wormhole.objects.filter.side_effect = [
            self._make_queryset([]),
            self._make_queryset([]),
            self._make_queryset([]),
        ]

        garbage_collection()

        self.assertEqual(mock_wormhole.objects.filter.call_count, 3)



from unittest.mock import Mock, patch

from django.forms import ModelChoiceField

from allianceauth.utils.testing import NoSocketsTestCase

from drifters.app_settings import DRIFTERS_JOVE_OBSERVATORIES
from drifters.forms import WormholeForm


class TestWormholeForm(NoSocketsTestCase):
    @patch("drifters.forms.SolarSystem")
    def test_init_uses_region_filter(self, mock_solar_system) -> None:
        queryset = Mock()
        mock_solar_system.objects.filter.return_value = queryset
        region = Mock()

        form = WormholeForm(region=region)
        solar_system_field = form.fields["solar_system"]
        assert isinstance(solar_system_field, ModelChoiceField)

        self.assertEqual(solar_system_field.queryset, queryset.all.return_value)
        mock_solar_system.objects.filter.assert_called_once_with(
            name__in=DRIFTERS_JOVE_OBSERVATORIES,
            constellation__region=region,
        )

    @patch("drifters.forms.SolarSystem")
    def test_init_uses_solar_system_filter(self, mock_solar_system) -> None:
        queryset = Mock()
        mock_solar_system.objects.filter.return_value = queryset
        solar_system = Mock(id=301)

        form = WormholeForm(solar_system=solar_system)
        solar_system_field = form.fields["solar_system"]
        assert isinstance(solar_system_field, ModelChoiceField)

        self.assertEqual(solar_system_field.queryset, queryset.all.return_value)
        mock_solar_system.objects.filter.assert_called_once_with(id=301)

    @patch("drifters.forms.SolarSystem")
    def test_init_uses_default_observatory_filter(self, mock_solar_system) -> None:
        queryset = Mock()
        mock_solar_system.objects.filter.return_value = queryset

        form = WormholeForm()
        solar_system_field = form.fields["solar_system"]
        assert isinstance(solar_system_field, ModelChoiceField)

        self.assertEqual(solar_system_field.queryset, queryset.all.return_value)
        mock_solar_system.objects.filter.assert_called_once_with(name__in=DRIFTERS_JOVE_OBSERVATORIES)



from datetime import timedelta

from eve_sde.models import Region, SolarSystem
from solo.models import SingletonModel

from django.contrib.auth.models import User
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from drifters.app_settings import DRIFTERS_JOVE_OBSERVATORIES


class General(models.Model):
    """Meta model for app permissions"""

    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            ("basic_access", "Can view the simple web interface and get read only cogs"),
            ("scout_access", "Can use the management ui and save wormholes via cogs"),
            ("manage_access", "UNUSED"),
        )


class WormholePublicActiveManager(models.Manager):
    def get_queryset(self) -> models.QuerySet["Wormhole"]:
        return super().get_queryset().filter(archived=False, reserved=False)


class WormholePrivateActiveManager(models.Manager):
    def get_queryset(self) -> models.QuerySet["Wormhole"]:
        return super().get_queryset().filter(archived=False)


class Wormhole(models.Model):

    class Complexes(models.TextChoices):
        """"
        The Various Drifter Complexes that a hole can lead to
        """
        BARBICAN = 'Barbican', _("Barbican")
        CONFLUX = 'Conflux', _("Conflux")
        REDOUBT = 'Redoubt', _("Redoubt")
        SENTINEL = 'Sentinel', _("Sentinel")
        VIDETTE = 'Vidette', _("Vidette")
        THERA = 'Thera', _("Thera")
        TURNUR = "Turnur", _("Turnur")

    class Mass(models.TextChoices):
        """"
        The states of Mass remaining
        """
        FRESH = 'Fresh', _(
            "This wormhole has not yet had its stability significantly disrupted by ships passing through it")  # >50%
        REDUCED = 'Reduced', _(
            "This wormhole has had its stability reduced by ships passing through it, but not to a critical degree yet")  # <50, >10%
        CRIT = 'Critical', _(
            "This wormhole has had its stability critically disrupted by the mass of numerous ships passing through and is on the verge of collapse")  # <10%

    class Lifetime(models.TextChoices):
        """"
        The states of Time Remaining
        """
        FRESH = 'Fresh', _(
            "This wormhole has not yet begun its natural cycle of decay and should last at least another day")   # >24 Hours?
        DECAY = 'Decaying', _(
            "This wormhole is beginning to decay, and probably won't last another day")  # <24h
        EOL = 'EOL', _("This wormhole is reaching the end of its natural lifetime")  # <4h

    solar_system = models.ForeignKey(SolarSystem, verbose_name=_("Solar System"), on_delete=models.CASCADE, limit_choices_to={"name__in": DRIFTERS_JOVE_OBSERVATORIES})
    complex = models.CharField(_("Complex"), max_length=50, choices=Complexes.choices)
    mass = models.CharField(_("Mass Remaining"), max_length=50, choices=Mass.choices)
    bookmarked_k = models.BooleanField(_("K Space Bookmarked"))
    bookmarked_w = models.BooleanField(_("W Space Bookmarked"))

    lifetime = models.CharField(_("Lifetime Remaining"), max_length=50, choices=Lifetime.choices)
    eol_changed_at = models.DateTimeField(auto_now=False, auto_now_add=False, blank=True, null=True, help_text="Only set if a user set a hole as EOL")

    created_at = models.DateTimeField(auto_now=False, auto_now_add=True)
    created_by = models.ForeignKey(User, verbose_name=_(
        "Created By User"), on_delete=models.CASCADE, null=True, blank=True, related_name="+")

    updated_at = models.DateTimeField(auto_now=False, auto_now_add=True)
    updated_by = models.ForeignKey(User, verbose_name=_(
        "Update By User"), on_delete=models.CASCADE, null=True, blank=True, related_name="+")

    archived = models.BooleanField(_("Archived"), default=False)
    reserved = models.BooleanField(_("Not Published / Reserved for Fleets and WH Team Usage"), default=False)
    archived_at = models.DateTimeField(blank=True, null=True)
    archived_by = models.ForeignKey(User, verbose_name=_(
        "Archived By User"), on_delete=models.CASCADE, null=True, blank=True, related_name="+")

    active_public_holes = WormholePublicActiveManager()
    active_private_holes = WormholePrivateActiveManager()
    objects = models.Manager()  # Add default manager back in because i made custom ones ^

    class Meta:
        verbose_name = _("Wormhole")
        verbose_name_plural = _("Wormholes")

    def __str__(self) -> str:
        return f"{self.solar_system.name} - {self.complex}"

    def set_eol(self, user: User = None) -> bool:
        """
        Set a wormhole as EOL, updating the eol_changed_at timestamp

        :return: Set EOL? False if not set or already EOL
        :rtype: bool
        """
        if self.lifetime == 'Decaying':

            if user is not None:
                self.updated_by = user
                self.updated_at = timezone.now()
                self.eol_changed_at = timezone.now()

            self.lifetime = 'EOL'
            self.save()
            return True
        return False

    def set_archived(self, user: User = None) -> bool:
        """
        Archives a hole, may optionally mark who archive it. if None, System

        :return: Archived? False if not set or already EOL
        :rtype: bool
        """
        self.archived_at = timezone.now()
        if user is not None:
            self.archived_by = user
        else:
            pass
        self.archived = True
        self.save()
        return True

    @property
    def age(self) -> timedelta:
        return timezone.now() - self.created_at

    @property
    def time_since_update(self) -> timedelta:
        return timezone.now() - self.updated_at

    @property
    def time_since_eol(self) -> timedelta:
        return timezone.now() - self.eol_changed_at

    @property
    def formatted_lifetime(self) -> str:
        if self.lifetime in [self.Lifetime.DECAY, self.Lifetime.FRESH]:
            return f"{self.lifetime} {self.age.total_seconds() / timedelta(hours=16).total_seconds():.0%}"
        elif self.eol_changed_at is None and self.lifetime == 'EOL':
            return f"{self.lifetime} {self.age.total_seconds() / timedelta(hours=4).total_seconds():.0%}"
        elif self.eol_changed_at is not None and self.lifetime == 'EOL':
            return f"{self.lifetime} {self.time_since_eol.total_seconds() / timedelta(hours=4).total_seconds():.0%}"
        return "N/A"

    @property
    def formatted_lifetime_motd(self) -> str:
        string = ""
        if self.lifetime in [self.Lifetime.DECAY.value, self.Lifetime.FRESH.value]:
            if self.time_since_update <= timedelta(hours=4):
                string += """<font size="12" color="#ff00ff00">"""
            elif self.time_since_update <= timedelta(hours=8):
                string += """<font size="12" color="#ffffff00">"""
            else:
                string += """<font size="12" color="#ffff0000">"""
            string += f"{self.lifetime} {self.age.total_seconds() / timedelta(hours=16).total_seconds():.0%}</font>"
            return string

        if self.lifetime == self.Lifetime.EOL.value and self.eol_changed_at is None:
            if self.time_since_update <= timedelta(hours=4):
                string += """<font size="12" color="#ff00ff00">"""
            elif self.time_since_update <= timedelta(hours=8):
                string += """<font size="12" color="#ffffff00">"""
            else:
                string += """<font size="12" color="#ffff0000">"""
            string += f"{self.lifetime} {self.age.total_seconds() / timedelta(hours=4).total_seconds():.0%}</font>"
            return string
        else:
            if self.time_since_update <= timedelta(hours=1):
                string += """<font size="12" color="#ff00ff00">"""
            elif self.time_since_update <= timedelta(hours=2):
                string += """<font size="12" color="#ffffff00">"""
            else:
                string += """<font size="12" color="#ffff0000">"""
            string += f"{self.lifetime} {self.time_since_eol.total_seconds() / timedelta(hours=4).total_seconds():.0%}</font>"
            return string

    @property
    def region(self) -> Region:
        return self.solar_system.constellation.region

    @property
    def complex_id(self) -> int:
        if self.complex == 'Sentinel':
            return 31000001
        elif self.complex == 'Barbican':
            return 31000002
        elif self.complex == 'Vidette':
            return 31000003
        elif self.complex == 'Conflux':
            return 31000004
        elif self.complex == 'Thera':
            return 31000005
        elif self.complex == 'Redoubt':
            return 31000006
        return 0


class Clear(models.Model):

    system = models.ForeignKey(SolarSystem, verbose_name=_("Solar System"), on_delete=models.CASCADE)

    created_at = models.DateTimeField(auto_now_add=True)
    created_by = models.ForeignKey(User, verbose_name=_("Created By User"), on_delete=models.CASCADE, null=True, blank=True, related_name="+")

    updated_at = models.DateTimeField(auto_now=True)
    updated_by = models.ForeignKey(User, verbose_name=_("Updated By User"), on_delete=models.CASCADE, null=True, blank=True, related_name="+")

    class Meta:
        verbose_name = _("Clear System")
        verbose_name_plural = _("Clear Systems")

    def __str__(self) -> str:
        return f"{self.system.name} - CLEAR"

    @property
    def age(self) -> timedelta:
        return self.created_at - timezone.now()

    @property
    def time_since_update(self) -> timedelta:
        return timezone.now() - self.updated_at


class DriftersConfiguration(SingletonModel):
    POI_regions = models.ManyToManyField(Region, verbose_name=_("POI Regions to be highlighted in the Web UI"), related_name="+", blank=True)
    POI_systems = models.ManyToManyField(SolarSystem, verbose_name=_("POI Systems to be highlighted in the Web UI"), related_name="+", blank=True)
    motd_systems = models.ManyToManyField(SolarSystem, verbose_name=_("Systems to be included in the MOTD Generation"), related_name="+", blank=True)

    def __str__(self) -> str:
        return "Drifters Configuration"

    class Meta:  # pyright: ignore[reportIncompatibleVariableOverride]
        """
        Meta definitions
        """
        verbose_name = "Drifters Configuration"
        default_permissions = ()



# Generated by Django 4.0.10 on 2023-06-22 04:52

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('drifters', '0005_alter_wormhole_archived_at_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='clear',
            name='system',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='eveuniverse.evesolarsystem', verbose_name='Solar System'),
        ),
        migrations.AlterField(
            model_name='wormhole',
            name='system',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='eveuniverse.evesolarsystem', verbose_name='Solar System'),
        ),
    ]



from datetime import datetime, timedelta, timezone as dt_timezone
from unittest.mock import Mock, patch

from allianceauth.tests.auth_utils import AuthUtils
from allianceauth.utils.testing import NoSocketsTestCase

from drifters.models import Wormhole


class TestWormholeModel(NoSocketsTestCase):
    def test_set_eol_updates_decaying_hole(self) -> None:
        now = datetime(2026, 3, 25, 12, 0, tzinfo=dt_timezone.utc)
        user = AuthUtils.create_user("test-user", disconnect_signals=True)
        wormhole = Wormhole(lifetime=Wormhole.Lifetime.DECAY)
        wormhole.save = Mock()

        with patch("drifters.models.timezone.now", side_effect=[now, now]):
            result = wormhole.set_eol(user=user)

        self.assertTrue(result)
        self.assertEqual(wormhole.lifetime, Wormhole.Lifetime.EOL)
        self.assertEqual(wormhole.updated_by, user)
        self.assertEqual(wormhole.updated_at, now)
        self.assertEqual(wormhole.eol_changed_at, now)
        wormhole.save.assert_called_once_with()

    def test_set_eol_does_nothing_for_non_decaying_hole(self) -> None:
        wormhole = Wormhole(lifetime=Wormhole.Lifetime.FRESH)
        wormhole.save = Mock()

        result = wormhole.set_eol(user=Mock())

        self.assertFalse(result)
        self.assertEqual(wormhole.lifetime, Wormhole.Lifetime.FRESH)
        wormhole.save.assert_not_called()

    def test_set_archived_sets_fields_and_saves(self) -> None:
        now = datetime(2026, 3, 25, 12, 0, tzinfo=dt_timezone.utc)
        user = AuthUtils.create_user("archive-user", disconnect_signals=True)
        wormhole = Wormhole(lifetime=Wormhole.Lifetime.FRESH)
        wormhole.save = Mock()

        with patch("drifters.models.timezone.now", return_value=now):
            result = wormhole.set_archived(user=user)

        self.assertTrue(result)
        self.assertTrue(wormhole.archived)
        self.assertEqual(wormhole.archived_at, now)
        self.assertEqual(wormhole.archived_by, user)
        wormhole.save.assert_called_once_with()

    def test_formatted_lifetime_for_fresh_hole(self) -> None:
        now = datetime(2026, 3, 25, 12, 0, tzinfo=dt_timezone.utc)
        wormhole = Wormhole(
            lifetime=Wormhole.Lifetime.FRESH,
            created_at=now - timedelta(hours=8),
        )

        with patch("drifters.models.timezone.now", return_value=now):
            result = wormhole.formatted_lifetime

        self.assertEqual(result, "Fresh 50%")

    def test_formatted_lifetime_for_eol_without_eol_changed(self) -> None:
        now = datetime(2026, 3, 25, 12, 0, tzinfo=dt_timezone.utc)
        wormhole = Wormhole(
            lifetime=Wormhole.Lifetime.EOL,
            created_at=now - timedelta(hours=1),
            eol_changed_at=None,
        )

        with patch("drifters.models.timezone.now", return_value=now):
            result = wormhole.formatted_lifetime

        self.assertEqual(result, "EOL 25%")

    def test_formatted_lifetime_for_eol_with_eol_changed(self) -> None:
        now = datetime(2026, 3, 25, 12, 0, tzinfo=dt_timezone.utc)
        wormhole = Wormhole(
            lifetime=Wormhole.Lifetime.EOL,
            created_at=now - timedelta(hours=3),
            eol_changed_at=now - timedelta(hours=2),
        )

        with patch("drifters.models.timezone.now", return_value=now):
            result = wormhole.formatted_lifetime

        self.assertEqual(result, "EOL 50%")

    def test_formatted_lifetime_motd_decay_green(self) -> None:
        now = datetime(2026, 3, 25, 12, 0, tzinfo=dt_timezone.utc)
        wormhole = Wormhole(
            lifetime=Wormhole.Lifetime.DECAY,
            created_at=now - timedelta(hours=8),
            updated_at=now - timedelta(hours=2),
        )

        with patch("drifters.models.timezone.now", return_value=now):
            result = wormhole.formatted_lifetime_motd

        self.assertIn('color="#ff00ff00"', result)
        self.assertIn("Decaying 50%", result)

    def test_formatted_lifetime_motd_eol_with_changed_yellow(self) -> None:
        now = datetime(2026, 3, 25, 12, 0, tzinfo=dt_timezone.utc)
        wormhole = Wormhole(
            lifetime=Wormhole.Lifetime.EOL,
            created_at=now - timedelta(hours=3),
            updated_at=now - timedelta(hours=2),
            eol_changed_at=now - timedelta(hours=1),
        )

        with patch("drifters.models.timezone.now", return_value=now):
            result = wormhole.formatted_lifetime_motd

        self.assertIn('color="#ffffff00"', result)
        self.assertIn("EOL 25%", result)

    def test_set_archived_without_user(self) -> None:
        now = datetime(2026, 3, 25, 12, 0, tzinfo=dt_timezone.utc)
        wormhole = Wormhole(lifetime=Wormhole.Lifetime.FRESH)
        wormhole.save = Mock()

        with patch("drifters.models.timezone.now", return_value=now):
            result = wormhole.set_archived()

        self.assertTrue(result)
        self.assertTrue(wormhole.archived)
        self.assertIsNone(wormhole.archived_by)
        self.assertEqual(wormhole.archived_at, now)
        wormhole.save.assert_called_once_with()

    def test_complex_id_mapping(self) -> None:
        expected_ids = {
            Wormhole.Complexes.SENTINEL: 31000001,
            Wormhole.Complexes.BARBICAN: 31000002,
            Wormhole.Complexes.VIDETTE: 31000003,
            Wormhole.Complexes.CONFLUX: 31000004,
            Wormhole.Complexes.THERA: 31000005,
            Wormhole.Complexes.REDOUBT: 31000006,
            Wormhole.Complexes.TURNUR: 0,
        }

        for complex_name, expected in expected_ids.items():
            wormhole = Wormhole(complex=complex_name)
            self.assertEqual(wormhole.complex_id, expected)



# Generated by Django 4.2.7 on 2023-11-15 02:29

import django.db.models.manager
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('drifters', '0010_general_alter_wormhole_eol_changed_at_and_more'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='wormhole',
            managers=[
                ('active_public_holes', django.db.models.manager.Manager()),
            ],
        ),
    ]



# Generated by Django 4.2.6 on 2023-11-03 13:17

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('drifters', '0007_alter_clear_options_driftersconfiguration'),
    ]

    operations = [
        migrations.AddField(
            model_name='driftersconfiguration',
            name='motd_systems',
            field=models.ManyToManyField(related_name='+', to='eveuniverse.evesolarsystem', verbose_name='Systems to be included in the MOTD Generation'),
        ),
        migrations.AlterField(
            model_name='clear',
            name='created_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='Created By User'),
        ),
        migrations.AlterField(
            model_name='clear',
            name='updated_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='Updated By User'),
        ),
        migrations.AlterField(
            model_name='driftersconfiguration',
            name='POI_regions',
            field=models.ManyToManyField(related_name='+', to='eveuniverse.everegion', verbose_name='POI Regions to be highlighted in the Web UI'),
        ),
        migrations.AlterField(
            model_name='driftersconfiguration',
            name='POI_systems',
            field=models.ManyToManyField(related_name='+', to='eveuniverse.evesolarsystem', verbose_name='POI Systems to be highlighted in the Web UI'),
        ),
        migrations.AlterField(
            model_name='wormhole',
            name='archived',
            field=models.BooleanField(default=False, verbose_name='Archived'),
        ),
        migrations.AlterField(
            model_name='wormhole',
            name='archived_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='Archived By User'),
        ),
        migrations.AlterField(
            model_name='wormhole',
            name='complex',
            field=models.CharField(choices=[('Barbican', 'Barbican'), ('Conflux', 'Conflux'), ('Redoubt', 'Redoubt'), ('Sentinel', 'Sentinel'), ('Vidette', 'Vidette'), ('Thera', 'Thera'), ('Turnur', 'Turnur')], max_length=50, verbose_name='Complex'),
        ),
        migrations.AlterField(
            model_name='wormhole',
            name='created_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='Created By User'),
        ),
        migrations.AlterField(
            model_name='wormhole',
            name='updated_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='Update By User'),
        ),
    ]



import logging

from aadiscordbot.cogs.utils.decorators import has_perm
from discord import AutocompleteContext, Option
from discord.commands import SlashCommandGroup
from discord.embeds import Embed
from discord.ext import commands
from eve_sde.models.map import Region, SolarSystem
from rapidfuzz import fuzz, process
from routing.routing import route_length, route_path_gates, systems_range

from django.conf import settings
from django.utils import timezone

from allianceauth.services.modules.discord.models import DiscordUser

from drifters import __version__
from drifters.app_settings import DRIFTERS_JOVE_OBSERVATORIES
from drifters.models import Clear, Wormhole
from drifters.routing import include_drifters_driftercomplexes

logger = logging.getLogger(__name__)


def get_complex_name_from_id(id: int) -> str:
    if id == 31000001:
        return Wormhole.Complexes.SENTINEL
    elif id == 31000002:
        return Wormhole.Complexes.BARBICAN
    elif id == 31000003:
        return Wormhole.Complexes.VIDETTE
    elif id == 31000004:
        return Wormhole.Complexes.CONFLUX
    elif id == 31000005:
        return Wormhole.Complexes.THERA
    elif id == 31000006:
        return Wormhole.Complexes.REDOUBT
    elif id == 30002086:
        return Wormhole.Complexes.TURNUR
    return ""


class Drifters(commands.Cog):
    """
    Drifter Wormhole Mapping and Management
    From AA-Drifters
    """

    def __init__(self, bot):
        self.bot = bot

    drifter_commands = SlashCommandGroup(
        "drifters", "Drifter Wormholes", guild_ids=[int(settings.DISCORD_GUILD_ID)])

    async def search_jove_obervatories(self, ctx: AutocompleteContext) -> list[str]:
        """
        Returns a subset of solar systems, known to have jove observatories, that begin with the characters entered so far

        :param ctx: _description_
        :type ctx: AutocompleteContext
        :return: Fuzzy Searched list of solar systems as a list of strings.
        :rtype: list
        """
        return [i[0] for i in process.extract(ctx.value, DRIFTERS_JOVE_OBSERVATORIES, scorer=fuzz.WRatio, limit=10)]

    async def search_solar_systems(self, ctx: AutocompleteContext) -> list[str]:
        """
        Returns a subset of solar systems, that begin with the characters entered so far

        :param ctx: _description_
        :type ctx: AutocompleteContext
        :return: Fuzzy Searched list of solar systems as a list of strings.
        :rtype: list
        """
        return list(SolarSystem.objects.filter(name__icontains=ctx.value).values_list('name', flat=True)[:10])

    async def search_regions(self, ctx: AutocompleteContext) -> list[str]:
        """
        Returns a list of SolarSystems that begin with the characters entered so far

        :param ctx: _description_
        :type ctx: AutocompleteContext
        :return: _description_
        :rtype: list
        """
        return list(Region.objects.filter(name__icontains=ctx.value).values_list('name', flat=True)[:10])

    @drifter_commands.command(name="about", description="About the Discord Bot", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def about(self, ctx):
        """
        All about the bot
        """
        embed = Embed(title="AA Drifters")
        embed.description = "https://gitlab.com/tactical-supremacy/aa-drifters\nShvo please come back"
        embed.url = "https://gitlab.com/tactical-supremacy/aa-drifters"
        embed.set_thumbnail(url="https://images.evetech.net/types/34495/render?size=128")
        embed.set_footer(
            text="Developed for INIT and publicly available to encourage destruction by Ariel Rin")
        embed.add_field(
            name="Version", value=f"{__version__}", inline=False
        )

        return await ctx.respond(embed=embed)

    @drifter_commands.command(name="clear", description="Report a system as Clear of drifter holes", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def clear(
        self, ctx,
        system=Option(str, "Solar System", autocomplete=search_jove_obervatories),
    ):
        """
        Adds a Clear record and optionally tidies up any holes in the system
        """
        archived_holes = 0
        try:
            clear_report, created = Clear.objects.get_or_create(
                system=SolarSystem.objects.get(name=system))
            if created is True:
                clear_report.created_at = timezone.now()
                clear_report.created_by = DiscordUser.objects.get(uid=ctx.user.id).user
            else:
                clear_report.updated_at = timezone.now()
                clear_report.updated_by = DiscordUser.objects.get(uid=ctx.user.id).user
            clear_report.save()
        except SolarSystem.DoesNotExist as e:
            logger.exception(e)
            return await ctx.respond(f"System {system} not found")
        except DiscordUser.DoesNotExist as e:
            logger.exception(e)
            return await ctx.respond("User not Found")
        except Exception as e:
            logger.exception(e)

        try:
            archived_holes = Wormhole.active_private_holes.filter(system=SolarSystem.objects.get(name=system)).update(
                archived=True,
                archived_at=timezone.now(),
                archived_by=DiscordUser.objects.get(uid=ctx.user.id).user)
        except SolarSystem.DoesNotExist as e:
            logger.exception(e)
            return await ctx.respond(f"System {system} not found")
        except DiscordUser.DoesNotExist as e:
            logger.exception(e)
            return await ctx.respond("User not Found")
        except Exception as e:
            logger.exception(e)

        if created is True:
            return await ctx.respond(f"{system} Marked as Clear for the first time, {archived_holes} Wormholes archived")
        elif created is False:
            return await ctx.respond(f"{system} Updated as Clear, {archived_holes} Wormholes archived")
        else:
            return await ctx.respond(f"{system} Unable to confirm=, please try again. {archived_holes} Wormholes archived")

    @drifter_commands.command(name="add", description="Add a Drifter wormhole in a K-Space system", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def add_wormhole(
        self, ctx,
        system=Option(str, "Solar System", autocomplete=search_jove_obervatories),
        complex=Option(str, "Drifter Complex", choices=Wormhole.Complexes),
        mass=Option(str, "Mass Status", choices=Wormhole.Mass.values),
        lifetime=Option(str, "Life Remaining", choices=Wormhole.Lifetime.values),
        bookmarked_k=Option(bool, "K Space Bookmarked?", default='True', choices=["True", "False"]),
        bookmarked_w=Option(bool, "W Space Bookmarked?", default='True', choices=["True", "False"])
    ):
        """
        Adds a drifter hole record
        """
        await ctx.trigger_typing()

        try:
            Clear.objects.get(SolarSystem.objects.get(name=system)).delete()
        except Exception as e:
            logger.exception(e)
        try:
            Wormhole.objects.create(
                system=SolarSystem.objects.get(name=system),
                complex=complex,
                mass=mass,
                lifetime=lifetime,
                created_at=timezone.now(),
                created_by=DiscordUser.objects.get(uid=ctx.user.id).user,
                bookmarked_k=bookmarked_k,
                bookmarked_w=bookmarked_w,
            )
        except SolarSystem.DoesNotExist:
            return await ctx.respond(f"System:{system} not found")
        except DiscordUser.DoesNotExist:
            return await ctx.respond("User not Found")
        except Exception as e:
            logger.exception(e)

        return await ctx.respond(f"Saved {complex} hole in {system}, {mass}, {lifetime} <t:{timezone.now():.0}:R>")

    @drifter_commands.command(name="complex_list", description="List wormholes leading to a complex", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def list_complex_wormholes(
        self, ctx,
        complex=Option(str, "Drifter Complex", choices=Wormhole.Complexes.values)
    ):
        """
        list_complex_wormholes _summary_

        :param ctx: _description_
        :type ctx: _type_
        :param complex: _description_, defaults to Option(str, "Drifter Complex", choices=Wormhole.Complexes.values)
        :type complex: _type_, optional
        :return: _description_
        :rtype: _type_
        """
        embed = Embed(title=f"AA-Drifters: {complex}")

        for wormhole in Wormhole.active_public_holes.filter(complex=complex):
            embed.add_field(
                name=wormhole.system.name, value=f"Mass {wormhole.mass} Lifetime {wormhole.lifetime}\n Updated:<t:{wormhole.updated_at.timestamp():.0f}:R>", inline=False
            )

        return await ctx.respond(embed=embed)

    @drifter_commands.command(name="system_list", description="List wormholes in a system", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def list_system_wormholes(
        self, ctx,
        system=Option(str, "Solar System", autocomplete=search_jove_obervatories),
    ):
        """
        list_system_wormholes _summary_

        :param ctx: _description_
        :type ctx: _type_
        :param system: _description_, defaults to Option(str, "Solar System", autocomplete=search_jove_obervatories)
        :type system: _type_, optional
        :return: _description_
        :rtype: _type_
        """
        solar_system = SolarSystem.objects.get(name=system)

        embed = Embed(title=f"AA-Drifters: {system}")

        for wormhole in Wormhole.active_public_holes.filter(system=solar_system):
            embed.add_field(
                name=wormhole.complex, value=f"{wormhole.mass}, {wormhole.formatted_lifetime}\n Updated:<t:{wormhole.updated_at.timestamp():.0f}:R>", inline=False
            )

        return await ctx.respond(embed=embed)

    @drifter_commands.command(name="system_jumps_list", description="List wormholes of wormoles x jumps from system", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def list_system_jumps_wormholes(
        self, ctx,
        system=Option(str, "Solar System", autocomplete=search_solar_systems),
        jumps=Option(int, "Jumps", default=5, required=True)
    ):
        """_summary_

        Args:
            ctx (_type_): _description_
            system (_type_, optional): _description_. Defaults to Option(str, "Solar System", autocomplete=search_solar_systems).
            jumps (_type_, optional): _description_. Defaults to Option(int, required=True).
        """
        solar_system = SolarSystem.objects.get(name=system)

        embed = Embed(title=f"AA-Drifters: {solar_system}")

        for wormhole in Wormhole.active_public_holes.filter(system__in=systems_range(solar_system.id, jumps)):
            embed.add_field(
                name=wormhole.complex, value=f"{wormhole.mass}, {wormhole.formatted_lifetime}, Updated:<t:{wormhole.updated_at.timestamp():.0f}:R>", inline=False
            )

        return await ctx.respond(embed=embed)

    @drifter_commands.command(name="region_list", description="List wormholes in a Region", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def list_region_wormholes(
        self, ctx,
        region=Option(str, "Region", autocomplete=search_regions),
    ):
        """
        list_region_wormholes _summary_

        :param ctx: _description_
        :type ctx: _type_
        :param system: _description_, defaults to Option(str, "Region", autocomplete=search_regions)
        :type system: _type_, optional
        :return: _description_
        :rtype: _type_
        """
        region_obj = Region.objects.get(name=region)

        embed = Embed(title=f"AA-Drifters: {region}")

        for wormhole in Wormhole.active_public_holes.filter(system__constellation__eve_region=region_obj):
            embed.add_field(
                name=wormhole.complex, value=f"{wormhole.mass}, {wormhole.formatted_lifetime}, Updated:<t:{wormhole.updated_at.timestamp():.0f}:R>", inline=False
            )

        return await ctx.respond(embed=embed)

    @drifter_commands.command(name="region_status", description="Known Jove Observatories in a region and their Status", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def status_region(
        self, ctx,
        region=Option(str, "Region", autocomplete=search_regions),
    ):
        """
        list_region_wormholes _summary_

        :param ctx: _description_
        :type ctx: _type_
        :param system: _description_, defaults to Option(str, "Region", autocomplete=search_regions)
        :type system: _type_, optional
        :return: _description_
        :rtype: _type_
        """
        region_obj = Region.objects.get(name=region)
        embed = Embed(title=f"AA-Drifters: Status {region}")
        content = ""

        for system in SolarSystem.objects.filter(constellation__region=region_obj, name__in=DRIFTERS_JOVE_OBSERVATORIES):
            if Wormhole.active_public_holes.filter(system=system):
                for wormhole in Wormhole.active_public_holes.filter(system=system):
                    content += f"{system.name} - {wormhole.complex}, {wormhole.mass}, {wormhole.formatted_lifetime}, Updated:<t:{wormhole.updated_at.timestamp():.0f}:R>) \n"
            elif Clear.objects.filter(system=system):
                for clear in Clear.objects.filter(system=system):
                    content += f"{system.name} - Clear (<t:{clear.updated_at.timestamp():.0f}:R>)\n"
            else:
                content += f"{system.name} - UNKNOWN\n"
        embed.description = content

        return await ctx.respond(embed=embed)

    @drifter_commands.command(name="route", description="Generate a Route with Drifters included", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def route(
        self, ctx,
        from_system=Option(str, "Solar System", autocomplete=search_solar_systems),
        to_system=Option(str, "Solar System", autocomplete=search_solar_systems),
        reserved=Option(bool, "K Space Bookmarked?", default='False', choices=["True", "False"])
    ):
        from_solar_system = SolarSystem.objects.get(name=from_system)
        to_solar_system = SolarSystem.objects.get(name=to_system)

        embed = Embed(title=f"AA-Drifters: Route {from_solar_system} -> {to_solar_system} Jumps:{route_length(from_solar_system.id, to_solar_system.id, edges=include_drifters_driftercomplexes())}")

        if reserved is True and has_perm(ctx.user.id, "drifters.scout_access"):
            wh_filter = Wormhole.active_private_holes
            use_reserved = True
        else:
            wh_filter = Wormhole.active_public_holes
            use_reserved = False

        for step in route_path_gates(from_solar_system.id, to_solar_system.id, edges=include_drifters_driftercomplexes(use_reserved=use_reserved)):
            step_from, step_to, step_type = step
            if step_type == "drifter_k":
                wh = wh_filter.filter(system_id=step_from, complex=get_complex_name_from_id(step_to)).order_by("-created_at").first()
                system, created = SolarSystem.objects.get_or_create(id=step_from)
                embed.add_field(name=step_type, value=f"{system} -> {wh.complex}", inline=False)
            elif step_type == "drifter_w":
                wh = wh_filter.filter(system_id=step_to, complex=get_complex_name_from_id(step_from)).order_by("-created_at").first()
                system, created = SolarSystem.objects.get_or_create(id=step_to)
                embed.add_field(name=step_type, value=f"{wh.complex} -> {system}", inline=False)
            elif step_type == "jump_bridge":
                wh = wh_filter.filter(system_id=step_to, complex=get_complex_name_from_id(step_from)).order_by("-created_at").first()
                system, created = SolarSystem.objects.get_or_create(id=step_to)
                embed.add_field(name=step_type, value=f"{wh.complex} -> {system}", inline=False)
            elif step_type == "Stargate":
                from_system, created = SolarSystem.objects.get_or_create(id=step_from)
                to_system, created = SolarSystem.objects.get_or_create(id=step_to)
                embed.add_field(name=step_type, value=f"{from_system} -> {to_system}", inline=True)
            else:
                embed.add_field(name="Unknown", value="Unknown -> Unknown", inline=True)

        return await ctx.respond(embed=embed)


def setup(bot):
    bot.add_cog(Drifters(bot))



# Generated by Django 4.0.10 on 2023-05-31 04:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('drifters', '0004_wormhole_bookmarked_k_wormhole_bookmarked_w_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='wormhole',
            name='archived_at',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='wormhole',
            name='eol_changed_at',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



# Generated by Django 4.0.10 on 2023-05-30 12:55

import datetime

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('drifters', '0003_wormhole_archived_wormhole_archived_at_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='wormhole',
            name='bookmarked_k',
            field=models.BooleanField(default=True, verbose_name='K Space Bookmarked'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='wormhole',
            name='bookmarked_w',
            field=models.BooleanField(default=True, verbose_name='W Space Bookmarked'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='wormhole',
            name='eol_changed_at',
            field=models.DateTimeField(default=datetime.datetime(2023, 5, 30, 12, 55, 59, 850235)),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='wormhole',
            name='archived_at',
            field=models.DateTimeField(blank=True),
        ),
        migrations.AlterField(
            model_name='wormhole',
            name='updated_at',
            field=models.DateTimeField(auto_now_add=True),
        ),
    ]



from django.contrib import admin

from allianceauth.services.hooks import get_extension_logger

from drifters.models import Clear, DriftersConfiguration, Wormhole

logger = get_extension_logger(__name__)


@admin.register(Wormhole)
class WormholeAdmin(admin.ModelAdmin):
    list_display = ['__str__', 'mass', 'lifetime']
    list_filter = ['mass', 'lifetime', 'bookmarked_k', 'bookmarked_w', 'archived']


@admin.register(Clear)
class ClearAdmin(admin.ModelAdmin):
    list_display = ['__str__',]


@admin.register(DriftersConfiguration)
class DriftersConfigurationAdmin(admin.ModelAdmin):
    list_display = ['id',]
    filter_horizontal = [
        "POI_regions",
        "POI_systems",
        "motd_systems"]



# Generated by Django 4.2.6 on 2023-11-03 16:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('drifters', '0008_driftersconfiguration_motd_systems_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='wormhole',
            name='reserved',
            field=models.BooleanField(default=False, verbose_name='Not Published / Reserved for Fleets and WH Team Usage'),
        ),
    ]



# Generated by Django 4.2.28 on 2026-03-02 11:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('drifters', '0013_alter_clear_system_and_more'),
    ]

    operations = [
        migrations.RenameField(
            model_name='wormhole',
            old_name='system',
            new_name='solar_system',
        ),
    ]



# Generated by Django 4.2.28 on 2026-03-02 11:29

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eve_sde', '0017_remove_itemtype_market_group_id_raw'),
        ('drifters', '0012_alter_driftersconfiguration_poi_regions_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='clear',
            name='system',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='eve_sde.solarsystem', verbose_name='Solar System'),
        ),
        migrations.AlterField(
            model_name='driftersconfiguration',
            name='POI_regions',
            field=models.ManyToManyField(blank=True, related_name='+', to='eve_sde.region', verbose_name='POI Regions to be highlighted in the Web UI'),
        ),
        migrations.AlterField(
            model_name='driftersconfiguration',
            name='POI_systems',
            field=models.ManyToManyField(blank=True, related_name='+', to='eve_sde.solarsystem', verbose_name='POI Systems to be highlighted in the Web UI'),
        ),
        migrations.AlterField(
            model_name='driftersconfiguration',
            name='motd_systems',
            field=models.ManyToManyField(blank=True, related_name='+', to='eve_sde.solarsystem', verbose_name='Systems to be included in the MOTD Generation'),
        ),
        migrations.AlterField(
            model_name='wormhole',
            name='system',
            field=models.ForeignKey(limit_choices_to={'name__in': ['Bazadod', 'Chibi', 'Defsunun', 'Esubara', 'Fihrneh', 'Hoseen', 'Huna', 'Isid', 'Nema', 'Nouta', 'Pserz', 'Soza', 'Aivonen', 'Enaluri', 'Hykanima', 'Ikoskio', 'Immuri', 'Innia', 'Nisuwa', 'Onnamon', 'Raihbaka', 'Rakapas', 'Teskanen', 'Uchomida', 'Anka', 'Ayeroilen', 'Hakodan', 'Haras', 'Huola', 'Iesa', 'Kamela', 'Komaa', 'Kourmonen', 'Kuomi', 'Kurniainen', 'Myyhera', 'Netsalakka', 'Otelen', 'Sahtogas', 'Saidusairos', 'Saikamon', 'Sasiekko', 'Satalama', 'Sosala', 'Tuomuta', 'Uusanen', '2B7A-3', '4DTQ-K', '52G-NZ', 'AH-B84', 'BU-IU4', 'C-LP3N', 'C-VGYO', 'CX-1XF', 'I-7JR4', 'J52-BH', 'KMC-WI', 'LRWD-B', 'M-HU4V', 'NEH-CS', 'O-JPKH', 'P7Z-R3', 'Q-NJZ4', 'SVB-RE', 'T-Q2DD', 'W-4FA9', 'YG-82V', 'Z-K495', '0LY-W1', 'I6-SYN', 'JZ-B5Y', 'LMM7-L', 'M-CNUD', 'NG-M8K', 'U-QMOA', 'U93O-A', '25S-6P', '3GD6-8', '4-07MU', '7LHB-Z', '9-8GBA', 'AX-DOT', 'B-XJX4', 'E3-SDZ', 'EX-0LQ', 'F4R2-Q', 'F9E-KX', 'G-AOTH', 'GE-94X', 'I-8D0G', 'IS-R7P', 'JA-O6J', 'JWZ2-V', 'L7XS-5', 'OGL8-Q', 'SNFV-I', 'ZQ-Z3Y', 'Aikoro', 'Hasama', 'Hysera', 'Irjunen', 'Ishomilken', 'Isutaka', 'Kaunokka', 'Kausaaja', 'Komo', 'Konola', 'Kulelen', 'Kusomonmon', 'Laah', 'Onatoh', 'Sankkasen', 'Santola', 'Tama', 'Uotila', 'Yashunen', '28O-JY', '6-4V20', '77-KDQ', '8R-RTB', 'G8AD-C', 'MJYW-3', 'O-IVNH', 'OU-X3P', 'QA1-BT', 'X-M9ON', 'XT-R36', '1GT-MA', '5E-EZC', '5ED-4E', '5HN-D6', '87-1PM', 'CHP-76', 'DK0-N8', 'E-B957', 'E-BYOS', 'FV-YEA', 'FV1-RQ', 'HB-5L3', 'L-Z9NB', 'X-41DA', 'Y-RAW3', 'Afivad', 'Ana', 'Arbaz', 'Ardishapur Prime', 'Ashab', 'Avair', 'Boranai', 'Cailanar', 'Ebidan', 'Ebtesham', 'Erzoh', 'Esteban', 'Fabin', 'Faswiba', 'Galeh', 'Hadonoo', 'Hutian', 'Jambu', 'Joppaya', 'Knophtikoo', 'Murzi', 'Nadohman', 'Nakri', 'Nifshed', 'Nomash', 'Rimbah', 'Seiradih', 'Sharji', 'Shuria', 'Sobenah', 'Somouh', 'Valmu', 'Yeeramoun', 'Yuhelia', 'ES-UWY', 'G-G78S', 'J7A-UR', 'K-MGJ7', 'V7D-JD', 'Y-K50G', '33RB-O', '3JN9-Q', '5W3-DG', 'E-FIC0', 'MXX5-9', 'MZ1E-P', 'N-TFXK', 'S-DN5M', 'TXME-A', 'U-TJ7Y', 'XCBK-X', 'ZOYW-O', '8F-TK3', 'D-W7F0', 'F-9PXR', 'HM-XR2', 'IP6V-X', 'KBAK-I', 'M-SRKS', 'QY6-RK', 'YZ9-F6', 'Z3V-1W', 'ZXB-VC', 'Zaimeth', 'Ziona', 'Abha', 'Agha', 'Akhrad', 'Asabona', 'Asghatil', 'Bayuka', 'Bekirdod', 'Faspera', 'Fovihi', 'Hahyil', 'Hasateem', 'Ihal', 'Juddi', 'Kuharah', 'Moh', 'Nimambal', 'Ordize', 'Salashayama', 'Sasta', 'Sendaya', 'Serad', 'Shach', 'Shala', 'Shamahi', 'Shenela', 'Uplingur', 'Yuzier', '15U-JY', '6F-H3W', '7BX-6F', '7X-02R', '87XQ-0', 'A-HZYL', 'AC2E-3', 'CHA2-Q', 'G-UTHL', 'H-S80W', 'HMF-9D', 'I-CUVX', 'IGE-RI', 'JGOW-Y', 'KCT-0A', 'L-1SW8', 'P5-EFH', 'PXF-RF', 'Q-XEB3', 'R3W-XU', 'SPLE-Y', 'UAYL-F', 'V6-NY1', 'YRNJ-8', 'Z-YN5Y', 'Z9PP-H', '6EK-BV', 'C-PEWN', 'CR-0E5', 'D-FVI7', 'F-UVBV', 'G-4H4C', 'G-YZUX', 'G2-INZ', 'HHE5-L', 'IPX-H5', 'JAUD-V', 'NIZJ-0', 'P9F-ZG', 'PE-H02', 'Q1-R7K', 'R-ARKN', 'SN9S-N', 'YRV-MZ', '0-G8NO', '1KAW-T', '3-LJW3', '4NDT-W', '5J4K-9', '62O-UE', '7-A6XV', '9-02G0', '9N-0HF', 'BOZ1-O', 'D-0UI0', 'E51-JE', 'GC-LTF', 'GR-X26', 'I-9GI1', 'L-QQ6P', 'L8-WNE', 'NB-ALM', 'P7MI-T', 'PQRE-W', 'RZ-PIY', 'S-51XG', 'SY-UWN', 'T-0JWP', 'TYB-69', 'U-MFTL', 'XA5-TY', 'Z-2Y2Y', 'Arayar', 'Arzad', 'Dihra', 'Hakshma', 'Halenan', 'Itsyamil', 'Mendori', 'Nidebora', 'Odin', 'Raa', 'Roushzar', 'Sosan', 'Thakala', 'Adrel', 'Algogille', 'Andole', 'Arant', 'Arnon', 'Charmerout', 'Haine', 'Heydieles', 'Hulmate', 'Isenan', 'Mesybier', 'Vifrevaert', 'Wysalan', '1PF-BC', '3IK-7O', '4-QDIX', '89-JPE', '8KE-YS', '9P-870', 'CT8K-0', 'F69O-M', 'IRD-HU', 'KGT3-6', 'KMH-J1', 'RO-0PZ', 'WPV-JN', 'Amattens', 'Antollare', 'Athinard', 'Cherore', 'Leremblompes', 'Marosier', 'Mormoen', 'Mya', 'Scuelazyns', '0-ARFO', 'O1Y-ED', 'X36Y-G', '2-F3OE', '2UK4-N', '3-BADZ', '6O-XIO', 'CL-J9W', 'DUU1-K', 'I9-ZQZ', 'K-X5AX', 'OXC-UL', 'P8-BKO', 'PO-3QW', 'QK-CDG', 'U-BXU9', 'UB5Z-3', 'UD-VZW', 'X6-J6R', 'Z-PNIA', 'ZID-LE', 'Ahtulaima', 'Akkio', 'Akora', 'Jakanerva', 'Mahtista', 'Nomaa', 'Nuken', 'Obanen', 'Ohkunen', 'Osaa', 'Otomainen', 'Outuni', 'Shihuken', 'Ukkalen', 'Uoyonen', '39-DGG', '4-CUM5', '4K0N-J', '6L78-1', 'B-F1MI', 'BWF-ZZ', 'D-I9HJ', 'FDZ4-A', 'HKYW-T', 'K42-IE', 'L-TOFR', 'LX-ZOJ', 'O2O-2X', 'QKTR-L', 'SR-KBB', 'TDE4-H', 'TJM-JJ', 'WH-2EZ', 'Access', 'Central Point', 'Chamja', 'Dom-Aphis', 'Habu', 'Hadji', 'Hesarid', 'Madomi', 'Malma', 'Nasreri', 'New Eden', 'Ourapheh', 'Pashanai', 'Sibe', 'Sigga', 'Simela', 'Tahli', 'Vecamia', '4M-P1I', '7Q-8Z2', '8DL-CP', 'B-VIP9', 'DE71-9', 'F5-CGW', 'HB-1NJ', 'IX8-JB', 'N-RAEL', 'NE-3GR', 'QM-20X', 'QQ3-YI', 'R-ESG0', 'TET3-B', 'UNJ-GX', 'UT-UZB', 'V89M-R', 'Y4-GQV', 'YUY-LM', 'ZM-DNR', 'Abudban', 'Ameinaka', 'Bogelek', 'Bosboger', 'Eystur', 'Frarn', 'Krilmokenur', 'Larkugei', 'Obrolber', 'Otraren', 'Rokofur', 'Sist', '6-I162', 'AC-7LZ', 'CJNF-J', 'FRTC-5', 'NS2L-4', 'O7-7UX', 'R-ORB7', 'U9U-TQ', 'Y-C4AL', 'Y-FZ5N', 'Y19P-1', '01TG-J', '4OIV-X', '6E-MOW', 'DDI-B7', 'DY-40Z', 'E7VE-V', 'GU-9F4', 'PZMA-E', 'X-0CKQ', 'Z-N9IP', '3Q-VZA', '4-EFLU', '5NQI-E', '8-OZU1', 'EIH-IU', 'G9L-LP', 'GPD5-0', 'HD-AJ7', 'HPBE-D', 'J-RXYN', 'J7-BDX', 'JEQG-7', 'JQU-KY', 'R0-DMM', 'U104-3', 'Aharalel', 'Ansasos', 'Ardhis', 'Asrios', 'Bersyrim', 'Chanoun', 'Halibai', 'Hikansog', 'Hiremir', 'Ithar', 'Jennim', 'Miah', 'Oberen', 'Peyiri', 'Rayeret', 'Salah', 'Ustnia', '0-VG7A', '1ZF-PJ', '4CJ-AC', '5-2PQU', '5C-RPA', '88A-RA', 'B-II34', 'C1G-XC', 'EKPB-3', 'EUU-4N', 'F3-8X2', 'F39H-1', 'GDHN-K', 'GK5Z-T', 'HL-VZX', 'KD-KPR', 'L5-UWT', 'MOCW-2', 'N-O53U', 'N7-BIY', 'O-7LAI', 'RD-FWY', 'V-QXXK', 'VBPT-T', 'XQP-9C', 'ZO-4AR', 'Arzieh', 'Ashmarir', 'Aurejet', 'Dimoohan', 'Fanathor', 'Firbha', 'Gidali', 'Hishai', 'Kuhri', 'Moro', 'Nahrneder', 'Nakis', 'Parses', 'Reteka', 'Sassecho', 'Sazre', 'Seshala', 'Talidal', 'Tzashrah', 'Zahefeus', '6FS-CZ', '86L-9F', 'BM-VYZ', 'BVRQ-O', 'C3J0-O', 'EPCD-D', 'HD-HOZ', 'HPV-RJ', 'J-OAH2', 'JT2I-7', 'K76A-3', 'LE-67X', 'R1O-GN', 'SH6X-F', 'TA9T-P', 'Aivoli', 'Ajanen', 'Ekura', 'Hakonen', 'Isseras', 'Kirras', 'Korama', 'Oishami', 'Ossa', 'Rauntaka', 'Ruvas', 'Sotrentaira', 'Tartoken', 'Tunttaras', 'Vuorrassi', 'Yoma', 'Amdonen', 'Ami', 'Andrub', 'Annad', 'Choga', 'Fensi', 'Khabara', 'Kizama', 'Kor-Azor Prime', 'Leva', 'Masanuh', 'Mora', 'Nahol', 'Nahyeen', 'Nakregde', 'Nibainkier', 'Nishah', 'Ordion', 'Perbhe', 'Rannoze', 'Shaha', 'Shokal', 'Tralasa', 'Zatamaka', 'Zorenyen', 'Agtver', 'Aldilur', 'Ardar', 'Arifsdald', 'Arnstur', 'Aset', 'Asgeir', 'Atonder', 'Auner', 'Earwik', 'Egbonbet', 'Egmar', 'Eram', 'Erindur', 'Erstet', 'Erstur', 'Eszur', 'Eygfe', 'Fildar', 'Hakeri', 'Hilfhurmur', 'Hjoramold', 'Hodrold', 'Jondik', 'Lasleinur', 'Lirerim', 'Nakugard', 'Nedegulf', 'Sirekur', 'Situner', 'Tabbetzur', '1NZV-7', '7T-0QS', 'APES-G', 'DAI-SH', 'F-TQWO', 'GL6S-2', 'H-EBQG', 'HG-YEQ', 'IBOX-2', 'JA-G0T', 'N-YLOE', 'NBW-GD', 'NSI-MW', 'POQP-K', 'PWPY-4', 'QSCO-D', 'RUF3-O', 'S5W-1Z', 'VVB-QH', 'XTVZ-E', 'Y-XZA7', 'ZOPZ-6', '3-JG3X', '6U-MFQ', 'DGDT-3', 'HO4E-Q', 'IG-4OF', 'IO-R2S', 'JXQJ-B', 'KED-2O', 'MZLW-9', 'RJBC-I', 'U-RELP', 'U9SE-N', 'X-CYNC', 'X-Z4JW', 'XKM-DE', 'Y-770C', 'Altbrard', 'Gulfonodi', 'Hedaleolfarber', 'Hegfunden', 'Hrokkur', 'Illamur', 'Orien', 'Osvetur', 'Sakulda', 'Varigne', '7-8EOE', '99-0GS', '9MWZ-B', 'C-WPWH', 'DJK-67', 'H90-C9', 'KR8-27', 'L-AS00', 'M9-FIB', 'OKEO-X', 'X-EHHD', 'Z-7OK1', '0-4VQL', '2ULC-J', '2WU-XT', '4AZV-W', '4O-ZRI', '8-AA98', 'CNHV-M', 'J7X-VN', 'KGCF-5', 'MC4C-H', 'N-I024', 'QFRV-2', 'QHH-13', 'QOK-SX', 'TFPT-U', 'WIO-OL', 'XUPK-Z', 'YQM-P1', 'ZZK-VF', '0-WVQS', '0D-CHA', 'A-5M31', 'C0T-77', 'CT7-5V', 'F5FO-U', 'JKJ-VJ', 'T-Z6J2', '3PPT-9', 'ARBX-9', 'GQ2S-8', 'H8-ZTO', 'LD-2VL', 'O-MCZR', 'O-N589', '0-U2M4', '4-1ECP', '9-ZA4Z', 'A1F-22', 'BEG-RL', 'E-WMT7', 'H-29TM', 'J9A-BH', 'JPEZ-R', 'JZ-UQC', 'LW-YEW', 'M4-KX5', 'MJ-5F9', 'MTO2-2', 'O-QKSM', 'OP7-BP', 'PE-SAM', 'PFV-ZH', 'RY-2FX', 'SY-OLX', 'TAL1-3', 'VWES-Y', 'XU7-CH', '08S-39', 'E-DOF2', 'RYQC-I', 'TCAG-3', 'VQE-CN', 'Z-M5A1', 'Adiere', 'Adrallezoen', 'Alles', 'Augnais', 'Aunia', 'Balle', 'Bamiette', 'Basgerin', 'Colelie', 'Crielere', 'Croleur', 'Doussivitte', 'Dunraelare', 'Eglennaert', 'Fluekele', 'Gallareue', 'Iyen-Oursta', 'Lamadent', 'Lermireve', 'Pucherie', 'Sharuveil', 'Stegette', 'Unel', 'Algasienan', 'Alsavoinon', 'Amoen', 'Aubonnie', 'Aulbres', 'Cumemare', 'Dastryns', 'Evaulon', 'Gallusiene', 'Mercomesier', 'Moclinamaud', 'Oicx', 'Pain', '1QZ-Y9', '2-KPW6', '4E-EZS', '6-KPAB', '9PX2-F', 'F2-NXA', 'FYD-TO', 'GN-PDU', 'H5N-V7', 'IAK-JW', 'JLO-Z3', 'K7-LDX', 'LBC-AW', 'LQ-OAI', 'NSBE-L', 'Q-K2T7', 'Q-NA5H', 'U2-28D', 'V-F6DQ', 'WFFE-4', 'WV0D-1', 'X9V-15', '2V-CS5', '8P9-BM', '9UY4-H', 'D-6WS1', 'FSW-3C', 'GN7-XY', 'HP-6Z6', 'I-MGAB', 'K1Y-5H', 'MH9C-S', 'N-RMSH', 'OXIY-V', 'QBL-BV', 'QR-K85', 'TU-O0T', 'Aeter', 'Anckee', 'Maire', 'Odinesyn', 'Ogaria', 'Pertnineere', 'Stoure', 'Toustain', 'Weraroix', 'Yvelet', '2B-3M4', '2IGP-1', '373Z-7', '37S-KO', '42-UOW', '4XW2-D', '6QBH-S', 'A-GPTM', 'B-G1LG', 'CJF-1P', 'FV-SE8', 'G-ME2K', 'J-AYLV', 'JU-UYK', 'L6B-0N', 'LGK-VP', 'LGL-SD', 'O-CT8N', 'TL-T9Z', 'UKYS-5', 'WEQT-K', 'WHG2-7', 'X5O1-L', 'XFBE-T', 'Y1-UQ2', '12YA-2', '4-ABS8', '5ZXX-K', 'JC-YX8', 'KQK1-2', 'R-2R0G', 'RORZ-H', 'RZC-16', 'S-MDYI', 'U-INPD', 'UC3H-Y', 'UI-8ZE', 'WW-KGD', 'B9EA-G', 'C6C-K9', 'E-BFLT', 'ETO-OT', 'GTB-O4', 'H4X-0I', 'HIK-MC', 'K-XJJT', 'K-YL9T', 'KPI-OW', 'M-NP5O', 'OTJ9-E', 'P65-TA', 'RXA-W1', '0-XIDJ', '2PLH-3', '4RX-EE', '9-266Q', '92K-H2', '9IPC-E', 'AJCJ-1', 'EK2-ET', 'HBD-CC', 'K3JR-J', 'M2-2V1', 'N-Q5PW', 'N5Y-4N', 'OJ-CT4', 'Y6-HPG', '3D5K-R', '8B-SAJ', 'A-BO4V', 'A3-LOG', 'E-VKJV', 'ED-L9T', 'KEJY-U', 'MKD-O8', 'O3L-95', 'OGY-6D', 'RF-CN3', 'TV8-HS', 'UVHO-F', 'UYU-VV', 'W6V-VM', 'Z-UZZN', '0LTQ-C', '2G38-I', '5-FGQI', '5-T0PZ', '5-VKCN', '51-5XG', '57-YRU', '6-CZ49', '617I-I', '9U-TTJ', 'BY-S36', 'CIS-7X', 'DUV-5Y', 'EU0I-T', 'FD-MLJ', 'KFR-ZE', 'Poitot', 'T-LIWS', 'ZN0-SR', 'Abai', 'Adahum', 'Ahkour', 'Ahrosseas', 'Arakor', 'Chamume', 'Gyerzen', 'Hilaban', 'Hoona', 'Ivih', 'Kerepa', 'Lari', 'Mani', 'Paye', 'Pera', 'Phoren', 'Pimsu', 'Remoriu', 'Rumida', 'Safilbab', 'Saminer', 'Teshkat', 'Tew', 'Yeder', '3DR-CR', '5-O8B1', '6OYQ-Z', '7D-PAT', 'A1RR-M', 'EOY-BG', 'KMV-CQ', 'PNS7-J', 'Q1U-IU', 'S-1LIO', 'SZ6-TA', 'T-K10W', 'WV-0R2', 'ZK-YQ3', '0P-U0Q', '16AM-3', '3L3N-X', '9-980U', 'BW-WJ2', 'F-ZBO0', 'FE-6YQ', 'H-HWQR', 'JI1-SY', 'KW-OAM', 'MS1-KJ', 'NZW-ZO', 'PEK-8Z', 'T-AKQZ', 'WB-AYY', 'Y-EQ0C', 'Y-ORBJ', 'ZD1-Z2', 'ZO-P5K', 'C2X-M5', 'C8VC-S', 'F-749O', 'NJ4X-S', 'S8-NSQ', 'SH1-6P', 'V0DF-2', 'W-UQA5', 'WH-JCA', 'XD-TOV', '0MV-4W', '1N-FJ8', '1VK-6B', '1W-0KS', '5T-KM3', '6WW-28', '7-K5EL', 'BR-6XP', 'FS-RFL', 'H-NOU5', 'JZV-F4', 'LS9B-9', 'MGAM-4', 'MQ-O27', 'O-LR1H', 'P3EN-E', 'Q-R3GP', 'S-NJBB', 'VI2K-J', 'XSQ-TF', 'Z-8Q65', 'ZA0L-U', 'Adallier', 'Alenia', 'Arraron', 'Channace', 'Claulenne', 'Merolles', 'Raneilles', 'Scolluzer', 'Tierijev']}, on_delete=django.db.models.deletion.CASCADE, to='eve_sde.solarsystem', verbose_name='Solar System'),
        ),
    ]



# Generated by Django 4.2.6 on 2023-11-01 05:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('drifters', '0006_alter_clear_system_alter_wormhole_system'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='clear',
            options={'verbose_name': 'Clear System', 'verbose_name_plural': 'Clear Systems'},
        ),
        migrations.CreateModel(
            name='DriftersConfiguration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('POI_regions', models.ManyToManyField(related_name='+', to='eveuniverse.everegion', verbose_name='POI Regions')),
                ('POI_systems', models.ManyToManyField(related_name='+', to='eveuniverse.evesolarsystem', verbose_name='POI Systems')),
            ],
            options={
                'verbose_name': 'Drifters Configuration',
                'default_permissions': (),
            },
        ),
    ]



from eve_sde.models.map import SolarSystem

from django import forms

from drifters.app_settings import DRIFTERS_JOVE_OBSERVATORIES

from .models import Wormhole


class WormholeForm(forms.ModelForm):
    class Meta:
        model = Wormhole
        fields = ["solar_system", "complex", "mass", "lifetime", "bookmarked_k", "bookmarked_w"]

    def __init__(self, user=None, **kwargs):
        self.region = kwargs.pop('region', None)
        self.solar_system = kwargs.pop('solar_system', None)
        super().__init__(**kwargs)
        if self.region:
            self.fields['solar_system'].queryset = SolarSystem.objects.filter(name__in=DRIFTERS_JOVE_OBSERVATORIES, constellation__region=self.region)
        elif self.solar_system:
            self.fields['solar_system'].queryset = SolarSystem.objects.filter(id=self.solar_system.id)
        else:
            self.fields['solar_system'].queryset = SolarSystem.objects.filter(name__in=DRIFTERS_JOVE_OBSERVATORIES)



from drifters.app_settings import DRIFTERS_DRIFTERHOLE_WEIGHT
from drifters.models import Wormhole

# Helper functions for AA Routing


def include_drifters_driftercomplexes(weight: float = DRIFTERS_DRIFTERHOLE_WEIGHT, use_reserved:bool = False) -> list[tuple]:
    edges = []

    if use_reserved is True:
        wh_filter = Wormhole.active_private_holes
    else:
        wh_filter = Wormhole.active_public_holes

    for complex in Wormhole.Complexes:
        for wh in wh_filter.filter(complex=complex):
            edges.append((
                wh.system.id, wh.complex_id,
                {'p_shortest': weight, 'p_safest': weight, 'p_less_safe': weight, "type": "drifter_k"}))
            edges.append((
                wh.complex_id, wh.system.id,
                {'p_shortest': weight, 'p_safest': weight, 'p_less_safe': weight, "type": "drifter_w"}))
    return edges



# Generated by Django 4.0.10 on 2023-05-30 08:58

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Wormhole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('complex', models.CharField(choices=[('Barbican', 'Barbican'), ('Conflux', 'Conflux'), ('Redoubt', 'Redoubt'), ('Sentinel', 'Sentinel'), ('Vidette', 'Vidette')], max_length=50, verbose_name='Complex')),
                ('mass', models.CharField(choices=[('Fresh', 'This wormhole has not yet had its stability significantly disrupted by ships passing through it'), ('Reduced', 'This wormhole has had its stability reduced by ships passing through it, but not to a critical degree yet'), ('Critical', 'This wormhole has had its stability critically disrupted by the mass of numerous ships passing through and is on the verge of collapse')], max_length=50, verbose_name='Mass Remaining')),
                ('lifetime', models.CharField(choices=[('Fresh', 'This wormhole has not yet begun its natural cycle of decay and should last at least another day'), ('Decaying', "This wormhole is beginning to decay, and probably won't last another day"), ('EOL', 'This wormhole is reaching the end of its natural lifetime')], max_length=50, verbose_name='Lifetime Remaining')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='')),
                ('system', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='eveuniverse.evesolarsystem', verbose_name='')),
                ('updated_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='')),
            ],
            options={
                'verbose_name': 'Wormhole',
                'verbose_name_plural': 'Wormholes',
            },
        ),
    ]



from eve_sde.models import Region, SolarSystem
from routing.routing import route_length, systems_range

from django.contrib.auth.decorators import login_required, permission_required
from django.forms import modelformset_factory
from django.http import HttpResponse
from django.shortcuts import render
from django.template.loader import render_to_string
from django.utils import timezone

from allianceauth.services.hooks import get_extension_logger

from drifters.app_settings import (
    DRIFTERS_BOOKMARK_FOLDER_ID, DRIFTERS_BOOKMARK_FOLDER_NAME,
    DRIFTERS_JOVE_OBSERVATORIES, DRIFTERS_MOTD_CHANNEL_LINK,
    DRIFTERS_MOTD_CHANNEL_NAME, DRIFTERS_MOTD_SYSTEM_SEARCH_RANGE,
)
from drifters.forms import WormholeForm
from drifters.models import DriftersConfiguration, Wormhole

logger = get_extension_logger(__name__)


def render_complex(request, complex: Wormhole.Complexes) -> str:
    context = {
        "complex": complex,
        "wormholes": Wormhole.active_public_holes.filter(complex=complex),
        "poi_regions": DriftersConfiguration.get_solo().POI_regions.all(),
        "poi_systems": DriftersConfiguration.get_solo().POI_systems.all(),
    }
    return render_to_string("drifters/blocks/complex.html", context, request)


@login_required
@permission_required("drifters.basic_access")
def index(request) -> HttpResponse:
    context = {'complex_renders': []}
    for complex in Wormhole.Complexes:
        context['complex_renders'].append(render_complex(request, complex))
    return render(request, "drifters/index.html", context)


def generate_channel_motd_1() -> str:
    # Generate an ingame channel MOTD with links as needed
    return f"""<font size="13" color="#ff6868e1"><a href="{DRIFTERS_MOTD_CHANNEL_LINK}">{DRIFTERS_MOTD_CHANNEL_NAME}</a></font><font size="13" color="#bfffffff"> (this service will not be available 24/7)<br></font>
<font size="10" color="#ffff0000">--TRAVELs WITH THE WH METRO ONLY AT YOUR OWN RISK!--<br></font>
<font size="12" color="#ffffffff"><b><u>Instructions & Safety Advice:</b></u>
- online & connect to</font><font size="12" color="#ff00a99d"> <a href="bookmarkFolder:{DRIFTERS_BOOKMARK_FOLDER_ID}">{DRIFTERS_BOOKMARK_FOLDER_NAME}</a></font><font size="12" color="#ffffffff">
- burn to entry/exit location and jump through the correct wormhole!<br>- small & fast travel ships with MWD,MJD and cloak are highly recommended!
- Drifter Battleship spawn on the outside of the wormholes and you need to be careful and avoid them at all cost!
- Check the time since last update at the bottom!<br></font>
<font size="10" color="#ff00ff00">Less than 4h, Reliable</font>
<font size="10" color="#ffffff00">More than 4h, Possible/Unconfirmed</font>
<font size="10" color="#ffff0000">More Than 8H Ago, Less Likely<br></font>
<font size="10" color="#ffffffff">% is remaining theoretical max life</font>
<font size="12" color="#ffffffff"><b><u>Active Metro Lines:</b></u></font>
"""


def generate_channel_motd_2() -> str:
    # Generate an ingame channel MOTD with links as needed

    motd_lines = ""

    for complex in Wormhole.Complexes:
        # These three reset to ensure we only post a Complex with >1 System with >= 1 Hole
        motd_lines_per_complex = ""
        valid_hole_count = 0
        valid_system_count = 0
        if Wormhole.active_public_holes.filter(complex=complex).exists():  # Exit early if we have no Wormholes belonging to a Complex
            motd_lines_per_complex += f"""<br><font size="12" color="#ffffffff"><b>{complex}</b></font>"""
            for motd_system in DriftersConfiguration.get_solo().motd_systems.all():
                # Reset here to only add a System with >0 Holes
                motd_lines_per_complex_per_system = ""
                valid_hole_count = 0
                motd_lines_per_complex_per_system += f"""<br><font size="12" color="#ffffffff"><b>{motd_system}</b><br></font>"""
                for hole in Wormhole.active_public_holes.filter(system__in=systems_range(motd_system.id, DRIFTERS_MOTD_SYSTEM_SEARCH_RANGE, include_source=True), complex=complex):
                    motd_lines_per_complex_per_system += f"""<font size="12" color="#ffd98d00"><a href="showinfo:5//{hole.system.id}">{hole.system}</a></font><font size="12" color="#ffffffff"> {route_length(motd_system.id, hole.system.id)}J</font> {hole.formatted_lifetime_motd}<br>"""
                    valid_hole_count += 1
                if valid_hole_count >= 1:
                    motd_lines_per_complex += motd_lines_per_complex_per_system
                    valid_system_count += 1

        if valid_system_count > 1:
            motd_lines += motd_lines_per_complex

    return f"""{motd_lines}<font size="12" color="#bfffffff"><i>Updated {timezone.now()}<br></i></font>"""


@login_required
@permission_required("drifters.basic_access")
def motd(request) -> HttpResponse:
    context = {
        'motd_1': generate_channel_motd_1(),
        'motd_2': generate_channel_motd_2()
    }

    return render(request, "drifters/motd.html", context)


def scout_region_render(request, region: Region) -> str:
    four_hours_ago = timezone.datetime.now() - timezone.timedelta(hours=4)
    eight_hours_ago = timezone.datetime.now() - timezone.timedelta(hours=8)
    twelve_hours_ago = timezone.datetime.now() - timezone.timedelta(hours=12)
    count = Wormhole.active_private_holes.filter(system__constellation__region=region).count()
    update_older_than_4h = Wormhole.active_private_holes.filter(system__constellation__region=region).exclude(updated_at__range=(four_hours_ago,timezone.now())).count()
    update_older_than_8h = Wormhole.active_private_holes.filter(system__constellation__region=region).exclude(updated_at__range=(eight_hours_ago,timezone.now())).count()
    update_older_than_12h = Wormhole.active_private_holes.filter(system__constellation__region=region).exclude(updated_at__range=(twelve_hours_ago,timezone.now())).count()

    context = {
        'region': region,
        'count': count,
        'update_older_than_4h': update_older_than_4h,
        'update_older_than_8h': update_older_than_8h,
        'update_older_than_12h': update_older_than_12h,
    }
    return render_to_string("drifters/scout/blocks/scout_region.html", context, request)


def scout_complex_render(request, complex: Wormhole.Complexes) -> str:

    four_hours_ago = timezone.datetime.now() - timezone.timedelta(hours=4)
    eight_hours_ago = timezone.datetime.now() - timezone.timedelta(hours=8)
    twelve_hours_ago = timezone.datetime.now() - timezone.timedelta(hours=12)
    count = Wormhole.active_private_holes.filter(complex=complex).count()
    update_older_than_4h = Wormhole.active_private_holes.filter(complex=complex,).exclude(updated_at__range=(four_hours_ago,timezone.now())).count()
    update_older_than_8h = Wormhole.active_private_holes.filter(complex=complex,).exclude(updated_at__range=(eight_hours_ago,timezone.now())).count()
    update_older_than_12h = Wormhole.active_private_holes.filter(complex=complex,).exclude(updated_at__range=(twelve_hours_ago,timezone.now())).count()
    context = {
        'complex': complex,
        'count': count,
        'update_older_than_4h': update_older_than_4h,
        'update_older_than_8h': update_older_than_8h,
        'update_older_than_12h': update_older_than_12h,
    }
    return render_to_string("drifters/scout/blocks/scout_complex.html", context, request)


@login_required
@permission_required("drifters.scout_access")
def scout(request) -> HttpResponse:
    context = {
        'scout_region_renders': [],
        "scout_complex_renders": []}

    for region in Region.objects.filter(id__lt="11000000"):
        context['scout_region_renders'].append(scout_region_render(request, region))

    for complex in Wormhole.Complexes:
        context['scout_complex_renders'].append(scout_complex_render(request, complex))

    return render(request, "drifters/scout/index.html", context)


def scout_system_render(request, system: SolarSystem) -> str:
    WormholeFormSet = modelformset_factory(Wormhole, fields=["system", "complex", "mass", "lifetime", "bookmarked_k", "bookmarked_w"], extra=1, form=WormholeForm)
    formset = WormholeFormSet(queryset=Wormhole.active_private_holes.filter(system=system), initial=[{"system": system}], prefix=system.id, form_kwargs={'system': system})

    context = {
        'system': system,
        'formset': formset
    }
    return render_to_string("drifters/scout/blocks/scout_system.html", context, request)


@login_required
@permission_required("drifters.scout_access")
def scout_region(request, region_id) -> HttpResponse:
    region = Region.objects.get(id=region_id)
    context = {'scout_system_renders': []}

    WormholeFormSet = modelformset_factory(Wormhole, fields=["system", "complex", "mass", "lifetime", "bookmarked_k", "bookmarked_w"], extra=1, form=WormholeForm)

    if request.method == "POST":
        for system in SolarSystem.objects.filter(constellation__eve_region=region, name__in=DRIFTERS_JOVE_OBSERVATORIES):
            formset = WormholeFormSet(request.POST, queryset=Wormhole.active_private_holes.filter(system=system), initial=[{"system": system}], prefix=system.id, form_kwargs={'region': region})
            if formset.is_valid():
                formset.save()

    for system in SolarSystem.objects.filter(constellation__eve_region=region, name__in=DRIFTERS_JOVE_OBSERVATORIES):
        context['scout_system_renders'].append(scout_system_render(request, system))
    return render(request, "drifters/scout/scout_region.html", context)


@login_required
@permission_required("drifters.scout_access")
def scout_complex(request, complex) -> HttpResponse:
    WormholeFormSet = modelformset_factory(Wormhole, fields=["system", "complex", "mass", "lifetime", "bookmarked_k", "bookmarked_w"], extra=1, form=WormholeForm)

    if request.method == "POST":
        formset = WormholeFormSet(request.POST, queryset=Wormhole.active_private_holes.filter(complex=complex), initial=[{"complex": complex}])
        if formset.is_valid():
            formset.save()
    else:
        formset = WormholeFormSet(queryset=Wormhole.active_private_holes.filter(complex=complex), initial=[{"complex": complex}])

    context = {
        'complex': complex,
        'formset': formset
    }
    return render(request, "drifters/scout/scout_complex.html", context)



from datetime import timedelta

from celery import shared_task

from django.utils import timezone

from drifters.models import Wormhole


@shared_task()
def garbage_collection():
    for wormhole in Wormhole.objects.filter(created_at__lte=timezone.now() - timedelta(hours=16)):
        wormhole.set_archived()

    for wormhole in Wormhole.objects.filter(lifetime=Wormhole.Lifetime.EOL,
                                            eol_changed_at__lte=timezone.now() - timedelta(hours=4)):
        wormhole.set_archived()

    for wormhole in Wormhole.objects.filter(lifetime__in=[Wormhole.Lifetime.DECAY, Wormhole.Lifetime.FRESH],
                                            created_at__lte=timezone.now() - timedelta(hours=12)):
        wormhole.set_eol()



# Generated by Django 4.0.10 on 2023-05-30 12:19

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('drifters', '0002_clear'),
    ]

    operations = [
        migrations.AddField(
            model_name='wormhole',
            name='archived',
            field=models.BooleanField(blank=True, default=False, null=True, verbose_name='Archived'),
        ),
        migrations.AddField(
            model_name='wormhole',
            name='archived_at',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AddField(
            model_name='wormhole',
            name='archived_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name=''),
        ),
    ]



# Generated by Django 4.0.10 on 2023-05-30 10:16

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('drifters', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Clear',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='')),
                ('system', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='eveuniverse.evesolarsystem', verbose_name='')),
                ('updated_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='')),
            ],
            options={
                'verbose_name': 'Wormhole',
                'verbose_name_plural': 'Wormholes',
            },
        ),
    ]



from datetime import datetime, timezone as dt_timezone
from unittest.mock import Mock, patch

from allianceauth.utils.testing import NoSocketsTestCase

from drifters.views import generate_channel_motd_1, generate_channel_motd_2


class TestViewHelpers(NoSocketsTestCase):
    def test_generate_channel_motd_1_contains_core_sections(self) -> None:
        output = generate_channel_motd_1()

        self.assertIn("Active Metro Lines", output)
        self.assertIn("bookmarkFolder:", output)
        self.assertIn("TRAVELs WITH THE WH METRO", output)

    @patch("drifters.views.timezone.now")
    @patch("drifters.views.DriftersConfiguration")
    @patch("drifters.views.Wormhole")
    def test_generate_channel_motd_2_without_holes_only_footer(
        self,
        mock_wormhole: Mock,
        _mock_config: Mock,
        mock_now: Mock,
    ) -> None:
        now = datetime(2026, 3, 25, 13, 0, tzinfo=dt_timezone.utc)
        mock_now.return_value = now

        mock_wormhole.Complexes = ["Sentinel"]
        exists_qs = Mock()
        exists_qs.exists.return_value = False
        mock_wormhole.active_public_holes.filter.return_value = exists_qs

        output = generate_channel_motd_2()

        self.assertIn("Updated 2026-03-25 13:00:00+00:00", output)
        self.assertNotIn("Sentinel", output)

    @patch("drifters.views.route_length")
    @patch("drifters.views.systems_range")
    @patch("drifters.views.timezone.now")
    @patch("drifters.views.DriftersConfiguration")
    @patch("drifters.views.Wormhole")
    def test_generate_channel_motd_2_with_two_systems_adds_complex_section(
        self,
        mock_wormhole: Mock,
        mock_config: Mock,
        mock_now: Mock,
        mock_systems_range: Mock,
        mock_route_length: Mock,
    ) -> None:
        now = datetime(2026, 3, 25, 13, 0, tzinfo=dt_timezone.utc)
        mock_now.return_value = now
        mock_systems_range.return_value = [30000142, 30002187]
        mock_route_length.side_effect = [3, 5]

        mock_wormhole.Complexes = ["Sentinel"]

        system_a = Mock(id=30000142)
        system_a.__str__ = Mock(return_value="Jita")
        system_b = Mock(id=30002187)
        system_b.__str__ = Mock(return_value="Amarr")

        hole_a = Mock(system=system_a, formatted_lifetime_motd="<font>Fresh 70%</font>")
        hole_b = Mock(system=system_b, formatted_lifetime_motd="<font>EOL 10%</font>")

        exists_qs = Mock()
        exists_qs.exists.return_value = True
        mock_wormhole.active_public_holes.filter.side_effect = [exists_qs, [hole_a], [hole_b]]

        mock_config.get_solo.return_value.motd_systems.all.return_value = [system_a, system_b]

        output = generate_channel_motd_2()

        self.assertIn("Sentinel", output)
        self.assertIn("Jita", output)
        self.assertIn("Amarr", output)
        self.assertIn("3J", output)
        self.assertIn("5J", output)



from django.urls import path

from drifters import views

app_name = "drifters"

urlpatterns = [
    path("", views.index, name="index"),
    path("motd", views.motd, name="motd"),
    path("scout", views.scout, name="scout"),
    path("scout/region/<int:region_id>/", views.scout_region, name="scout_region"),
    path("scout/complex/<str:complex>/", views.scout_complex, name="scout_complex"),
]



from eve_sde.models import SolarSystem
from routing.routing import systems_range

from django import template

from drifters.app_settings import DRIFTERS_POI_SEARCH_RANGE
from drifters.models import Wormhole

register = template.Library()


@register.simple_tag()
def find_closest_holes(system: SolarSystem, complex: Wormhole.Complexes):
    holes = Wormhole.active_public_holes.filter(system__in=systems_range(system.id, DRIFTERS_POI_SEARCH_RANGE, include_source=True), complex=complex)
    if holes.count() == 0:
        return None
    else:
        return holes



# Generated by Django 4.2.16 on 2024-11-26 11:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('drifters', '0011_alter_wormhole_managers'),
    ]

    operations = [
        migrations.AlterField(
            model_name='driftersconfiguration',
            name='POI_regions',
            field=models.ManyToManyField(blank=True, related_name='+', to='eveuniverse.everegion', verbose_name='POI Regions to be highlighted in the Web UI'),
        ),
        migrations.AlterField(
            model_name='driftersconfiguration',
            name='POI_systems',
            field=models.ManyToManyField(blank=True, related_name='+', to='eveuniverse.evesolarsystem', verbose_name='POI Systems to be highlighted in the Web UI'),
        ),
        migrations.AlterField(
            model_name='driftersconfiguration',
            name='motd_systems',
            field=models.ManyToManyField(blank=True, related_name='+', to='eveuniverse.evesolarsystem', verbose_name='Systems to be included in the MOTD Generation'),
        ),
    ]
